@@ -12,12 +12,20 @@ GType cut_test_result_status_get_type (void);
 #define CUT_TYPE_TEST_RESULT_STATUS (cut_test_result_status_get_type())
 GType cut_order_get_type (void);
 #define CUT_TYPE_ORDER (cut_order_get_type())
+GType cut_test_context_error_get_type (void);
+#define CUT_TYPE_TEST_CONTEXT_ERROR (cut_test_context_error_get_type())
+GType cut_pipeline_error_get_type (void);
+#define CUT_TYPE_PIPELINE_ERROR (cut_pipeline_error_get_type())
 GType cut_verbose_level_get_type (void);
 #define CUT_TYPE_VERBOSE_LEVEL (cut_verbose_level_get_type())
 GType cut_verbose_level_error_get_type (void);
 #define CUT_TYPE_VERBOSE_LEVEL_ERROR (cut_verbose_level_error_get_type())
 GType cut_sequence_match_operation_type_get_type (void);
 #define CUT_TYPE_SEQUENCE_MATCH_OPERATION_TYPE (cut_sequence_match_operation_type_get_type())
+GType cut_stream_reader_error_get_type (void);
+#define CUT_TYPE_STREAM_READER_ERROR (cut_stream_reader_error_get_type())
+GType cut_file_stream_reader_error_get_type (void);
+#define CUT_TYPE_FILE_STREAM_READER_ERROR (cut_file_stream_reader_error_get_type())
 G_END_DECLS
 
 #endif /* __CUT_ENUM_TYPES_H__ */
