@@ -18,7 +18,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#  include <config.h>
 #endif /* HAVE_CONFIG_H */
 
 #include <stdlib.h>
@@ -319,6 +319,7 @@ cut_repository_collect_loader (CutRepository *repository, const gchar *dir_name,
             } else {
                 priv->loaders = g_list_prepend(priv->loaders, loader);
             }
+            g_free(relative_path);
         }
         g_free(path_name);
     }
