@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2007-2009  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2007-2009  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -112,9 +112,6 @@ gboolean      cut_test_context_is_failed      (CutTestContext *context);
 void          cut_test_context_set_jump       (CutTestContext *context,
                                                jmp_buf        *buffer);
 jmp_buf      *cut_test_context_get_jump       (CutTestContext *context);
-void          cut_test_context_set_backtrace  (CutTestContext *context,
-                                               const gchar    *backtrace);
-const gchar  *cut_test_context_get_backtrace  (CutTestContext *context);
 void          cut_test_context_emit_signal    (CutTestContext *context,
                                                CutTestResult  *result);
 
