@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2007-2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2007-2009  Kouhei Sutou <kou@cozmixng.org>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -18,7 +18,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#  include <config.h>
 #endif /* HAVE_CONFIG_H */
 
 #include <stdlib.h>
@@ -276,16 +276,10 @@ set_property (GObject      *object,
       case PROP_USER_MESSAGE:
         cut_test_result_set_user_message(CUT_TEST_RESULT(object),
                                          g_value_get_string(value));
-        if (priv->message)
-            g_free(priv->message);
-        priv->message = NULL;
         break;
       case PROP_SYSTEM_MESSAGE:
         cut_test_result_set_system_message(CUT_TEST_RESULT(object),
                                            g_value_get_string(value));
-        if (priv->message)
-            g_free(priv->message);
-        priv->message = NULL;
         break;
       case PROP_BACKTRACE:
         cut_test_result_set_backtrace(CUT_TEST_RESULT(object),
@@ -557,20 +551,23 @@ result_status_to_name (CutTestResultStatus status)
       case CUT_TEST_RESULT_SUCCESS:
         return "success";
         break;
+      case CUT_TEST_RESULT_NOTIFICATION:
+        return "notification";
+        break;
+      case CUT_TEST_RESULT_OMISSION:
+        return "omission";
+        break;
+      case CUT_TEST_RESULT_PENDING:
+        return "pending";
+        break;
       case CUT_TEST_RESULT_FAILURE:
         return "failure";
         break;
       case CUT_TEST_RESULT_ERROR:
         return "error";
         break;
-      case CUT_TEST_RESULT_PENDING:
-        return "pending";
-        break;
-      case CUT_TEST_RESULT_NOTIFICATION:
-        return "notification";
-        break;
-      case CUT_TEST_RESULT_OMISSION:
-        return "omission";
+      case CUT_TEST_RESULT_CRASH:
+        return "crash";
         break;
       default:
         return "unknown status";
@@ -685,6 +682,9 @@ cut_test_result_status_to_signal_name (CutTestResultStatus status)
       case CUT_TEST_RESULT_ERROR:
         signal_name = "error";
         break;
+      case CUT_TEST_RESULT_CRASH:
+        signal_name = "crash";
+        break;
       default:
         signal_name = "invalid status";
         break;
@@ -781,8 +781,12 @@ cut_test_result_set_user_message (CutTestResult *result,
         g_free(priv->user_message);
         priv->user_message = NULL;
     }
-    if (user_message)
+    if (user_message && user_message[0])
         priv->user_message = g_strdup(user_message);
+
+    if (priv->message)
+        g_free(priv->message);
+    priv->message = NULL;
 }
 
 void
@@ -809,8 +813,12 @@ cut_test_result_set_system_message (CutTestResult *result,
         g_free(priv->system_message);
         priv->system_message = NULL;
     }
-    if (system_message)
+    if (system_message && system_message[0])
         priv->system_message = g_strdup(system_message);
+
+    if (priv->message)
+        g_free(priv->message);
+    priv->message = NULL;
 }
 
 void
