@@ -18,7 +18,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#  include <config.h>
 #endif /* HAVE_CONFIG_H */
 
 #include <glib.h>
@@ -77,7 +77,7 @@ constructor (GType type, guint n_props, GObjectConstructParam *props)
         object = klass->constructor(type, n_props, props);
         the_builder = CUT_UI_FACTORY_BUILDER(object);
     } else {
-        object = g_object_ref(G_OBJECT(the_builder));
+        object = g_object_ref(the_builder);
     }
 
     return object;
