@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2007-2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2007-2009  Kouhei Sutou <kou@cozmixng.org>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -18,7 +18,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#  include <config.h>
 #endif /* HAVE_CONFIG_H */
 
 #include <string.h>
@@ -28,6 +28,9 @@
 #endif
 #include <glib.h>
 #include <glib/gstdio.h>
+#ifdef G_OS_WIN32
+#  include <windows.h>
+#endif
 
 #include <errno.h>
 
@@ -36,8 +39,10 @@
 #include "cut-sub-process-group.h"
 #include "cut-diff.h"
 #include "cut-main.h"
+#include "cut-backtrace-entry.h"
 #include "../gcutter/gcut-public.h"
 #include "../gcutter/gcut-error.h"
+#include "../gcutter/gcut-assertions-helper.h"
 
 gchar *
 cut_utils_create_regex_pattern (const gchar *string)
@@ -105,16 +110,34 @@ cut_utils_inspect_memory (const void *memory, size_t size)
 {
     const guchar *binary = memory;
     GString *buffer;
-    size_t i;
+    size_t i, n_printable_characters;
 
     if (memory == NULL || size == 0)
         return g_strdup("(null)");
 
-    buffer = g_string_sized_new(size * 5);
+    buffer = g_string_sized_new(size * strlen("0xXX") +
+                                (size - 1) * strlen(" ") +
+                                strlen(": ") +
+                                size);
+    n_printable_characters = 0;
     for (i = 0; i < size; i++) {
         g_string_append_printf(buffer, "0x%02x ", binary[i]);
+        if (g_ascii_isprint(binary[i]))
+            n_printable_characters++;
+    }
+
+    if (n_printable_characters >= size * 0.3) {
+        g_string_overwrite(buffer, buffer->len - 1, ": ");
+        for (i = 0; i < size; i++) {
+            if (g_ascii_isprint(binary[i])) {
+                g_string_append_c(buffer, binary[i]);
+            } else {
+                g_string_append_c(buffer, '.');
+            }
+        }
+    } else {
+        g_string_truncate(buffer, buffer->len - 1);
     }
-    g_string_truncate(buffer, buffer->len - 1);
 
     return g_string_free(buffer, FALSE);
 }
@@ -131,6 +154,19 @@ cut_utils_equal_string (const gchar *string1, const gchar *string2)
     return g_str_equal(string1, string2);
 }
 
+gboolean
+cut_utils_equal_substring (const gchar *string1, const gchar *string2,
+                           size_t length)
+{
+    if (string1 == string2)
+        return TRUE;
+
+    if (string1 == NULL || string2 == NULL)
+        return FALSE;
+
+    return strncmp(string1, string2, length) == 0;
+}
+
 gboolean
 cut_utils_equal_double (gdouble double1, gdouble double2, gdouble error)
 {
@@ -205,60 +241,53 @@ cut_utils_regex_match (const gchar *pattern, const gchar *string)
     return g_regex_match_simple(pattern, string, G_REGEX_MULTILINE, 0);
 }
 
+gchar *
+cut_utils_regex_replace (const gchar *pattern, const gchar *string,
+                         const gchar *replacement, GError **error)
+{
+    GRegex *regex;
+    gchar *replaced;
+
+    regex = g_regex_new(pattern, G_REGEX_MULTILINE, 0, error);
+    if (!regex)
+        return NULL;
+
+    replaced = g_regex_replace(regex, string, -1, 0, replacement, 0, error);
+    g_regex_unref(regex);
+
+    return replaced;
+}
+
 const gchar *
 cut_utils_get_fixture_data_string (CutTestContext *context,
+                                   gchar **fixture_data_path,
                                    const char *path,
                                    ...)
 {
-    GError *error = NULL;
     const gchar *data;
     va_list args;
 
     va_start(args, path);
-    data = cut_test_context_get_fixture_data_stringv(context, &error,
-                                                     path, &args);
-    va_end(args);
+    data = cut_utils_get_fixture_data_string_va_list(context, fixture_data_path,
+                                                     path, args);
+    va_end(args); /* FIXME: should be freed */
 
-    if (error) {
-        gchar *inspected, *message;
-
-        inspected = gcut_error_inspect(error);
-        message = g_strdup_printf("can't get fixture data: %s", inspected);
-        g_error_free(error);
-        cut_test_context_register_result(context,
-                                         CUT_TEST_RESULT_ERROR,
-                                         message, NULL);
-        g_free(inspected);
-        g_free(message);
-        cut_test_context_long_jump(context);
-    }
     return data;
 }
 
-void
-cut_utils_get_fixture_data_string_and_path (CutTestContext *context,
-                                            gchar **data,
-                                            gchar **fixture_data_path,
-                                            const gchar *path,
-                                            ...)
+const gchar *
+cut_utils_get_fixture_data_string_va_list (CutTestContext *context,
+                                           gchar **fixture_data_path,
+                                           const gchar *path,
+                                           va_list args)
 {
     GError *error = NULL;
-    va_list args, copied_args;
     const gchar *fixture_data;
 
-    va_start(args, path);
-
-    va_copy(copied_args, args);
-    *fixture_data_path = cut_test_context_build_fixture_data_pathv(context,
-                                                                   path,
-                                                                   &copied_args);
-    va_end(copied_args);
-
-    fixture_data = cut_test_context_get_fixture_data_stringv(context, &error,
-                                                             path, &args);
-    *data = (gchar *)fixture_data;
-    va_end(args);
-
+    fixture_data =
+        cut_test_context_get_fixture_data_string_va_list(context, &error,
+                                                         fixture_data_path,
+                                                         path, args);
     if (error) {
         gchar *inspected, *message;
 
@@ -269,11 +298,13 @@ cut_utils_get_fixture_data_string_and_path (CutTestContext *context,
         g_error_free(error);
         cut_test_context_register_result(context,
                                          CUT_TEST_RESULT_ERROR,
-                                         message, NULL);
+                                         message);
         g_free(inspected);
         g_free(message);
         cut_test_context_long_jump(context);
     }
+
+    return fixture_data;
 }
 
 void
@@ -383,14 +414,14 @@ cut_utils_build_path (const gchar *path, ...)
     va_list args;
 
     va_start(args, path);
-    built_path = cut_utils_build_pathv(path, &args);
+    built_path = cut_utils_build_path_va_list(path, args);
     va_end(args);
 
     return built_path;
 }
 
 gchar *
-cut_utils_build_pathv (const gchar *path, va_list *args)
+cut_utils_build_path_va_list (const gchar *path, va_list args)
 {
     GArray *elements;
     gchar *element, *concatenated_path;
@@ -401,7 +432,7 @@ cut_utils_build_pathv (const gchar *path, va_list *args)
     elements = g_array_new(TRUE, FALSE, sizeof(char *));
 
     g_array_append_val(elements, path);
-    while ((element = va_arg(*args, gchar *))) {
+    while ((element = va_arg(args, gchar *))) {
         g_array_append_val(elements, element);
     }
     concatenated_path = g_build_filenamev((gchar **)(elements->data));
@@ -430,14 +461,14 @@ cut_utils_expand_path (const gchar *path)
 }
 
 gchar *
-cut_utils_expand_pathv (const gchar *path, va_list *args)
+cut_utils_expand_path_va_list (const gchar *path, va_list args)
 {
     gchar *concatenated_path, *expanded_path;
 
     if (!path)
         return NULL;
 
-    concatenated_path = cut_utils_build_pathv(path, args);
+    concatenated_path = cut_utils_build_path_va_list(path, args);
     expanded_path = cut_utils_expand_path(concatenated_path);
     g_free(concatenated_path);
     return expanded_path;
@@ -475,6 +506,7 @@ cut_utils_remove_path_recursive (const char *path, GError **error)
     if (g_file_test(path, G_FILE_TEST_IS_DIR)) {
         GDir *dir;
         const gchar *name;
+        gboolean success = TRUE;
 
         dir = g_dir_open(path, 0, error);
         if (!dir)
@@ -482,23 +514,21 @@ cut_utils_remove_path_recursive (const char *path, GError **error)
 
         while ((name = g_dir_read_name(dir))) {
             gchar *full_path;
-            gboolean success;
 
             full_path = g_build_filename(path, name, NULL);
             success = cut_utils_remove_path_recursive(full_path, error);
             g_free(full_path);
             if (!success)
-                return FALSE;
+                break;
         }
 
         g_dir_close(dir);
 
-        return cut_utils_remove_path(path, error);
-    } else {
-        return cut_utils_remove_path(path, error);
+        if (!success)
+            return FALSE;
     }
 
-    return TRUE;
+    return cut_utils_remove_path(path, error);
 }
 
 void
@@ -595,6 +625,109 @@ cut_utils_take_new_sub_process_group (CutTestContext *test_context)
     return group;
 }
 
+typedef enum
+{
+    GDB_BACKTRACE_START,
+    GDB_BACKTRACE_IN,
+    GDB_BACKTRACE_FUNCTION,
+    GDB_BACKTRACE_ARGUMENTS,
+    GDB_BACKTRACE_ARGUMENTS_END,
+    GDB_BACKTRACE_AT,
+    GDB_BACKTRACE_LINE
+} GdbBacktraceState;
+
+GList *
+cut_utils_parse_gdb_backtrace (const gchar *gdb_backtrace)
+{
+    GList *backtraces = NULL;
+
+    while (gdb_backtrace && gdb_backtrace[0]) {
+        CutBacktraceEntry *entry;
+        gchar *file = NULL;
+        guint line = 0;
+        gchar *function = NULL;
+        gchar *info = NULL;
+        const gchar *start_point = NULL;
+        GdbBacktraceState state = GDB_BACKTRACE_START;
+
+        while (gdb_backtrace[0] && gdb_backtrace[0] != '\n') {
+            switch (state) {
+            case GDB_BACKTRACE_START:
+                if (gdb_backtrace[0] == 'i' && gdb_backtrace[1] == 'n') {
+                    gdb_backtrace++;
+                    state = GDB_BACKTRACE_IN;
+                }
+                break;
+            case GDB_BACKTRACE_IN:
+                if (start_point) {
+                    if (gdb_backtrace[0] == ' ') {
+                        function = g_strndup(start_point,
+                                             gdb_backtrace - start_point);
+                        state = GDB_BACKTRACE_ARGUMENTS;
+                        start_point = NULL;
+                    }
+                } else {
+                    if (gdb_backtrace[0] != ' ')
+                        start_point = gdb_backtrace;
+                }
+                break;
+            case GDB_BACKTRACE_ARGUMENTS:
+                if (gdb_backtrace[0] == ')')
+                    state = GDB_BACKTRACE_ARGUMENTS_END;
+                break;
+            case GDB_BACKTRACE_ARGUMENTS_END:
+                if (gdb_backtrace[0] == 'a' && gdb_backtrace[1] == 't') {
+                    gdb_backtrace ++;
+                    state = GDB_BACKTRACE_AT;
+                }
+                break;
+            case GDB_BACKTRACE_AT:
+                if (start_point) {
+                    if (gdb_backtrace[0] == ':') {
+                        file = g_strndup(start_point,
+                                         gdb_backtrace - start_point);
+                        state = GDB_BACKTRACE_LINE;
+                        start_point = NULL;
+                    }
+                } else {
+                    if (gdb_backtrace[0] != ' ')
+                        start_point = gdb_backtrace;
+                }
+                break;
+            case GDB_BACKTRACE_LINE:
+                if (start_point) {
+                    if (gdb_backtrace[1] == '\n') {
+                        line = atoi(start_point);
+                        start_point = NULL;
+                    }
+                } else {
+                    start_point = gdb_backtrace;
+                }
+                break;
+            default:
+                break;
+            }
+            gdb_backtrace++;
+        }
+
+        entry = cut_backtrace_entry_new(file ? file : "unknown",
+                                        line,
+                                        function,
+                                        info);
+        backtraces = g_list_append(backtraces, entry);
+        if (file)
+            g_free(file);
+        if (function)
+            g_free(function);
+        if (info)
+            g_free(info);
+
+        gdb_backtrace++;
+    }
+
+    return backtraces;
+}
+
 #ifdef G_OS_WIN32
 static gchar *win32_base_path = NULL;
 
@@ -604,7 +737,7 @@ cut_win32_base_path (void)
     if (win32_base_path)
         return win32_base_path;
 
-    win32_base_path = g_win32_get_package_installation_directory(PACKAGE, NULL);
+    win32_base_path = g_win32_get_package_installation_directory_of_module(NULL);
 
     return win32_base_path;
 }
@@ -626,6 +759,12 @@ cut_win32_build_factory_module_dir_name (const gchar *type)
     g_free(module_dir);
     return factory_module_dir;
 }
+
+gboolean
+cut_win32_kill_process (GPid pid, guint exit_code)
+{
+    return TerminateProcess(pid, exit_code) ? TRUE : FALSE;
+}
 #endif
 
 
