@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2007-2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2007-2009  Kouhei Sutou <kou@cozmixng.org>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -35,7 +35,7 @@ extern "C" {
 /**
  * SECTION: cutter
  * @title: Cutter
- * @short_description: An Unit Testing Framework for C.
+ * @short_description: A Unit Testing Framework for C.
  * @see_also: <link linkend="cutter-cut-assertions">Assertions</link>
  *
  * Cutter is a Unit Testing Framework for C and has the
@@ -135,44 +135,79 @@ extern "C" {
  * debugging.
  *   </para></listitem>
  * </itemizedlist>
-  */
+ */
 
+#ifndef CUTTER_DISABLE_DEPRECATED
 /**
  * setup:
  *
  * If you define setup() in your test program, cutter will
- * call your setup() before each your test is run.
+ * call your setup() before each your test is run. If you
+ * define cut_setup(), setup() is ignored.
+ *
+ * Deprecated: 1.0.7: Use cut_setup() instead.
  */
 void setup(void);
+#endif
 
+/**
+ * cut_setup:
+ *
+ * If you define cut_setup() in your test program, cutter
+ * will call your cut_setup() before each your test is
+ * run. cut_setup() has priority over setup().
+ *
+ * Since: 1.0.6
+ */
+void cut_setup(void);
+
+#ifndef CUTTER_DISABLE_DEPRECATED
 /**
  * teardown:
  *
  * If you define teardown() in your test program, cutter will
  * call your teardown() after each your test is run even if
- * a test is failed.
+ * a test is failed. If you define cut_teardown(),
+ * teardown() is ignored.
+ *
+ * Deprecated: 1.0.7: Use cut_teardown() instead.
  */
 void teardown(void);
+#endif
 
 /**
- * startup:
+ * cut_teardown:
  *
- * If you define startup() in your test program, cutter will
- * call your startup() before each your test case is run.
+ * If you define cut_teardown() in your test program, cutter
+ * will call your cut_teardown() after each your test is run
+ * even if a test is failed. cut_teardown() has priority
+ * over teardown().
  *
- * Since: 0.8
+ * Since: 1.0.6
  */
-void startup(void);
+void cut_teardown(void);
 
 /**
- * shutdown:
+ * cut_startup:
  *
- * If you define shutdown() in your test program, cutter will
- * call your shutdown() after each your test case is run.
+ * If you define cut_startup() in your test program, cutter
+ * will call your cut_startup() before each your test case
+ * is run. cut_startup() has priority over startup().
  *
- * Since: 0.8
+ * Since: 1.0.6
  */
-void shutdown(void);
+void cut_startup(void);
+
+/**
+ * cut_shutdown:
+ *
+ * If you define shutdown() in your test program, cutter
+ * will call your shutdown() after each your test case is
+ * run.  cut_shutdown() has priority over shutdown().
+ *
+ * Since: 1.0.6
+ */
+void cut_shutdown(void);
 
 /**
  * cut_add_data:
@@ -180,7 +215,8 @@ void shutdown(void);
  * @...: The data and destroy function of the first data,
  *       followed optionally by more
  *       name/data/destroy_function(#CutDestroyFunction)
- *       triples.
+ *       triples. The variable arguments should be terminated
+ *       by %NULL since 1.0.6.
  *
  * Adds data to use data driven test.
  *
@@ -237,7 +273,8 @@ void shutdown(void);
  *                  translate_test_data_free,
  *                  "complex data",
  *                  translate_test_data_new("a hundred eleven", 111),
- *                  translate_test_data_free);
+ *                  translate_test_data_free,
+ *                  NULL);
  * }
  *
  * void
@@ -252,16 +289,24 @@ void shutdown(void);
  *
  * Since: 1.0.3
  */
-#define cut_add_data(first_data_name, ...)                      \
-    cut_test_context_add_data(cut_get_current_test_context(),   \
-                              first_data_name, ## __VA_ARGS__,  \
-                              NULL)
+#define cut_add_data(first_data_name, ...)                              \
+    cut_add_data_backward_compatibility(cut_get_current_test_context(), \
+                                        first_data_name, __VA_ARGS__)
+
+#if !defined(CUTTER_DISABLE_DEPRECATED) && defined(__GNUC__)
+#define cut_add_data_backward_compatibility(context, ...)       \
+    cut_test_context_add_data(context, ## __VA_ARGS__, NULL)
+#else
+#define cut_add_data_backward_compatibility(context, ...)       \
+    cut_test_context_add_data(context, __VA_ARGS__)
+#endif
 
 /**
  * cut_set_attributes:
  * @first_attribute_name: The first attribute name.
  * @...: The value of the first attribute, followed
  *       optionally by more name/value pairs.
+ *       %NULL-terminate is required since 1.0.7.
  *
  * Sets attributes of the test.
  *
@@ -277,7 +322,8 @@ void shutdown(void);
  * {
  *     cut_set_attributes("description", "a test for repeat function",
  *                        "bug", "111",
- *                        "priority", "high");
+ *                        "priority", "high",
+ *                        NULL);
  * }
  *
  * void
@@ -292,8 +338,7 @@ void shutdown(void);
 #define cut_set_attributes(first_attribute_name, ...)                   \
     cut_test_context_set_attributes(cut_get_current_test_context(),     \
                                     first_attribute_name,               \
-                                    ## __VA_ARGS__,                     \
-                                    NULL)
+                                    __VA_ARGS__)
 
 /**
  * cut_get_current_test_context:
@@ -313,7 +358,7 @@ void shutdown(void);
  * {
  *     CutTestContext *test_context = data;
  *     cut_set_current_test_context(test_context);
- *     ....
+ *     ...
  * }
  *
  * void
@@ -334,7 +379,7 @@ void shutdown(void);
  * Since: 1.0.4
  */
 #define cut_get_current_test_context()          \
-    cut_test_context_current_get()
+    cut_test_context_current_peek()
 
 /**
  * cut_set_current_test_context:
@@ -347,8 +392,44 @@ void shutdown(void);
  * Since: 1.0.4
  */
 #define cut_set_current_test_context(test_context)      \
-    cut_test_context_current_set(test_context)
+    cut_test_context_current_push(test_context)
 
+/**
+ * cut_set_message:
+ * @format: the message format. See the printf() documentation.
+ * @...: the parameters to insert into the format string.
+ *
+ * Sets a message to be used by the next assertion.
+ *
+ * Since: 1.0.6
+ */
+#define cut_set_message(...)                                            \
+    cut_test_context_set_user_message(cut_get_current_test_context(),   \
+                                      __VA_ARGS__)
+
+/**
+ * cut_set_message_va_list:
+ * @format: the message format. See the printf() documentation.
+ * @args: the parameters to insert into the format string.
+ *
+ * Sets a message to be used by the next assertion.
+ *
+ * Since: 1.0.6
+ */
+#define cut_set_message_va_list(format, args)           \
+    cut_test_context_set_user_message_va_list(          \
+        cut_get_current_test_context(), format, args)
+
+/**
+ * cut_keep_message:
+ *
+ * Keeps the current message set by cut_set_message() or
+ * cut_set_message_va_list() after the next assertion.
+ *
+ * Since: 1.0.6
+ */
+#define cut_keep_message()                                              \
+    cut_test_context_keep_user_message(cut_get_current_test_context())
 
 #ifdef __cplusplus
 }
