@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2009  Kouhei Sutou <kou@cozmixng.org>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -18,7 +18,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#  include <config.h>
 #endif /* HAVE_CONFIG_H */
 
 #include <string.h>
@@ -28,27 +28,25 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 
+#include <cutter/cut-utils.h>
 #include "gcut-assertions-helper.h"
 
 void
 gcut_assert_equal_type_helper (GType           expected,
                                GType           actual,
                                const gchar    *expression_expected,
-                               const gchar    *expression_actual,
-                               const gchar    *user_message_format,
-                               ...)
+                               const gchar    *expression_actual)
 {
     if (expected == actual) {
         cut_test_pass();
     } else {
-        cut_test_fail_va_list(cut_take_printf("<%s == %s>\n"
-                                              "expected: <%s>\n"
-                                              "  actual: <%s>",
-                                              expression_expected,
-                                              expression_actual,
-                                              g_type_name(expected),
-                                              g_type_name(actual)),
-                              user_message_format);
+        cut_test_fail(cut_take_printf("<%s == %s>\n"
+                                      "expected: <%s>\n"
+                                      "  actual: <%s>",
+                                      expression_expected,
+                                      expression_actual,
+                                      g_type_name(expected),
+                                      g_type_name(actual)));
     }
 }
 
@@ -56,9 +54,7 @@ void
 gcut_assert_equal_value_helper (GValue         *expected,
                                 GValue         *actual,
                                 const gchar    *expression_expected,
-                                const gchar    *expression_actual,
-                                const gchar    *user_message_format,
-                                ...)
+                                const gchar    *expression_actual)
 {
     if (gcut_value_equal(expected, actual)) {
         cut_test_pass();
@@ -79,7 +75,44 @@ gcut_assert_equal_value_helper (GValue         *expected,
                                   inspected_expected, expected_type_name,
                                   inspected_actual, actual_type_name);
         message = cut_append_diff(message, inspected_expected, inspected_actual);
-        cut_test_fail_va_list(message, user_message_format);
+        cut_test_fail(message);
+    }
+}
+
+void
+gcut_assert_equal_list_helper (const GList    *expected,
+                               const GList    *actual,
+                               GEqualFunc      equal_function,
+                               GCutInspectFunction inspect_function,
+                               gpointer        inspect_user_data,
+                               const gchar    *expression_expected,
+                               const gchar    *expression_actual,
+                               const gchar    *expression_equal_function)
+{
+    if (gcut_list_equal(expected, actual, equal_function)) {
+        cut_test_pass();
+    } else {
+        const gchar *message;
+        const gchar *inspected_expected, *inspected_actual;
+
+        inspected_expected =
+            cut_take_string(gcut_list_inspect(expected,
+                                              inspect_function,
+                                              inspect_user_data));
+        inspected_actual =
+            cut_take_string(gcut_list_inspect(actual,
+                                              inspect_function,
+                                              inspect_user_data));
+
+        message = cut_take_printf("<%s(%s[i], %s[i]) == TRUE>\n"
+                                  "expected: <%s>\n"
+                                  "  actual: <%s>",
+                                  expression_equal_function,
+                                  expression_expected, expression_actual,
+                                  inspected_expected,
+                                  inspected_actual);
+        message = cut_append_diff(message, inspected_expected, inspected_actual);
+        cut_test_fail(message);
     }
 }
 
@@ -87,18 +120,16 @@ void
 gcut_assert_equal_list_int_helper (const GList    *expected,
                                    const GList    *actual,
                                    const gchar    *expression_expected,
-                                   const gchar    *expression_actual,
-                                   const gchar    *user_message_format,
-                                   ...)
+                                   const gchar    *expression_actual)
 {
-    if (gcut_list_int_equal(expected, actual)) {
+    if (gcut_list_equal_int(expected, actual)) {
         cut_test_pass();
     } else {
         const gchar *message;
         const gchar *inspected_expected, *inspected_actual;
 
-        inspected_expected = cut_take_string(gcut_list_int_inspect(expected));
-        inspected_actual = cut_take_string(gcut_list_int_inspect(actual));
+        inspected_expected = cut_take_string(gcut_list_inspect_int(expected));
+        inspected_actual = cut_take_string(gcut_list_inspect_int(actual));
 
         message = cut_take_printf("<%s == %s>\n"
                                   "expected: <%s>\n"
@@ -107,7 +138,7 @@ gcut_assert_equal_list_int_helper (const GList    *expected,
                                   inspected_expected,
                                   inspected_actual);
         message = cut_append_diff(message, inspected_expected, inspected_actual);
-        cut_test_fail_va_list(message, user_message_format);
+        cut_test_fail(message);
     }
 }
 
@@ -115,18 +146,16 @@ void
 gcut_assert_equal_list_uint_helper (const GList    *expected,
                                     const GList    *actual,
                                     const gchar    *expression_expected,
-                                    const gchar    *expression_actual,
-                                    const gchar    *user_message_format,
-                                    ...)
+                                    const gchar    *expression_actual)
 {
-    if (gcut_list_uint_equal(expected, actual)) {
+    if (gcut_list_equal_uint(expected, actual)) {
         cut_test_pass();
     } else {
         const gchar *message;
         const gchar *inspected_expected, *inspected_actual;
 
-        inspected_expected = cut_take_string(gcut_list_uint_inspect(expected));
-        inspected_actual = cut_take_string(gcut_list_uint_inspect(actual));
+        inspected_expected = cut_take_string(gcut_list_inspect_uint(expected));
+        inspected_actual = cut_take_string(gcut_list_inspect_uint(actual));
 
         message = cut_take_printf("<%s == %s>\n"
                                   "expected: <%s>\n"
@@ -135,7 +164,7 @@ gcut_assert_equal_list_uint_helper (const GList    *expected,
                                   inspected_expected,
                                   inspected_actual);
         message = cut_append_diff(message, inspected_expected, inspected_actual);
-        cut_test_fail_va_list(message, user_message_format);
+        cut_test_fail(message);
     }
 }
 
@@ -143,27 +172,25 @@ void
 gcut_assert_equal_list_string_helper (const GList    *expected,
                                       const GList    *actual,
                                       const gchar    *expression_expected,
-                                      const gchar    *expression_actual,
-                                      const gchar    *user_message_format,
-                                      ...)
+                                      const gchar    *expression_actual)
 {
-    if (gcut_list_string_equal(expected, actual)) {
+    if (gcut_list_equal_string(expected, actual)) {
         cut_test_pass();
     } else {
         const gchar *message;
         const gchar *inspected_expected, *inspected_actual;
 
-        inspected_expected = cut_take_string(gcut_list_string_inspect(expected));
-        inspected_actual = cut_take_string(gcut_list_string_inspect(actual));
+        inspected_expected = cut_take_string(gcut_list_inspect_string(expected));
+        inspected_actual = cut_take_string(gcut_list_inspect_string(actual));
 
         message = cut_take_printf("<%s == %s>\n"
                                   "expected: <%s>\n"
                                   "  actual: <%s>",
                                   expression_expected, expression_actual,
                                   inspected_expected,
-                                         inspected_actual);
+                                  inspected_actual);
         message = cut_append_diff(message, inspected_expected, inspected_actual);
-        cut_test_fail_va_list(message, user_message_format);
+        cut_test_fail(message);
     }
 }
 
@@ -173,20 +200,18 @@ gcut_assert_equal_list_enum_helper (GType           type,
                                     const GList    *actual,
                                     const gchar    *expression_type,
                                     const gchar    *expression_expected,
-                                    const gchar    *expression_actual,
-                                    const gchar    *user_message_format,
-                                    ...)
+                                    const gchar    *expression_actual)
 {
-    if (gcut_list_equal(expected, actual, g_direct_equal)) {
+    if (gcut_list_equal_int(expected, actual)) {
         cut_test_pass();
     } else {
         const gchar *message;
         const gchar *inspected_expected, *inspected_actual;
 
         inspected_expected =
-            cut_take_string(gcut_list_enum_inspect(expected, type));
+            cut_take_string(gcut_list_inspect_enum(type, expected));
         inspected_actual =
-            cut_take_string(gcut_list_enum_inspect(actual, type));
+            cut_take_string(gcut_list_inspect_enum(type, actual));
 
         message = cut_take_printf("<%s == %s>\n"
                                   "expected: <%s>\n"
@@ -195,7 +220,7 @@ gcut_assert_equal_list_enum_helper (GType           type,
                                   inspected_expected,
                                   inspected_actual);
         message = cut_append_diff(message, inspected_expected, inspected_actual);
-        cut_test_fail_va_list(message, user_message_format);
+        cut_test_fail(message);
     }
 }
 
@@ -205,20 +230,18 @@ gcut_assert_equal_list_flags_helper (GType           type,
                                      const GList    *actual,
                                      const gchar    *expression_type,
                                      const gchar    *expression_expected,
-                                     const gchar    *expression_actual,
-                                     const gchar    *user_message_format,
-                                     ...)
+                                     const gchar    *expression_actual)
 {
-    if (gcut_list_equal(expected, actual, g_direct_equal)) {
+    if (gcut_list_equal_uint(expected, actual)) {
         cut_test_pass();
     } else {
         const gchar *message;
         const gchar *inspected_expected, *inspected_actual;
 
         inspected_expected =
-            cut_take_string(gcut_list_flags_inspect(expected, type));
+            cut_take_string(gcut_list_inspect_flags(type, expected));
         inspected_actual =
-            cut_take_string(gcut_list_flags_inspect(actual, type));
+            cut_take_string(gcut_list_inspect_flags(type, actual));
 
         message = cut_take_printf("<%s == %s>\n"
                                   "expected: <%s>\n"
@@ -227,7 +250,7 @@ gcut_assert_equal_list_flags_helper (GType           type,
                                   inspected_expected,
                                   inspected_actual);
         message = cut_append_diff(message, inspected_expected, inspected_actual);
-        cut_test_fail_va_list(message, user_message_format);
+        cut_test_fail(message);
     }
 }
 
@@ -236,9 +259,7 @@ gcut_assert_equal_list_object_helper (const GList    *expected,
                                       const GList    *actual,
                                       GEqualFunc      equal_function,
                                       const gchar    *expression_expected,
-                                      const gchar    *expression_actual,
-                                      const gchar    *user_message_format,
-                                      ...)
+                                      const gchar    *expression_actual)
 {
     if (gcut_list_equal(expected, actual, equal_function)) {
         cut_test_pass();
@@ -247,9 +268,9 @@ gcut_assert_equal_list_object_helper (const GList    *expected,
         const gchar *inspected_expected, *inspected_actual;
 
         inspected_expected =
-            cut_take_string(gcut_list_object_inspect(expected));
+            cut_take_string(gcut_list_inspect_object(expected));
         inspected_actual =
-            cut_take_string(gcut_list_object_inspect(actual));
+            cut_take_string(gcut_list_inspect_object(actual));
 
         message = cut_take_printf("<%s == %s>\n"
                                   "expected: <%s>\n"
@@ -258,18 +279,54 @@ gcut_assert_equal_list_object_helper (const GList    *expected,
                                   inspected_expected,
                                   inspected_actual);
         message = cut_append_diff(message, inspected_expected, inspected_actual);
-        cut_test_fail_va_list(message, user_message_format);
+        cut_test_fail(message);
     }
 }
 
 void
-gcut_assert_equal_hash_table_string_string_helper (
-    GHashTable     *expected,
-    GHashTable     *actual,
-    const gchar    *expression_expected,
-    const gchar    *expression_actual,
-    const gchar    *user_message_format,
-    ...)
+gcut_assert_equal_hash_table_helper (GHashTable  *expected,
+                                     GHashTable  *actual,
+                                     GEqualFunc   equal_function,
+                                     GCutInspectFunction key_inspect_function,
+                                     GCutInspectFunction value_inspect_function,
+                                     gpointer     inspect_user_data,
+                                     const gchar *expression_expected,
+                                     const gchar *expression_actual,
+                                     const gchar *expression_equal_function)
+{
+    if (gcut_hash_table_equal(expected, actual, equal_function)) {
+        cut_test_pass();
+    } else {
+        const gchar *message;
+        gchar *inspected_expected, *inspected_actual;
+
+        inspected_expected = gcut_hash_table_inspect(expected,
+                                                     key_inspect_function,
+                                                     value_inspect_function,
+                                                     inspect_user_data);
+        inspected_actual = gcut_hash_table_inspect(actual,
+                                                   key_inspect_function,
+                                                   value_inspect_function,
+                                                   inspect_user_data);
+        message = cut_take_printf("<%s(%s[key], %s[key]) == TRUE>\n"
+                                  "expected: <%s>\n"
+                                  "  actual: <%s>",
+                                  expression_equal_function,
+                                  expression_expected, expression_actual,
+                                  inspected_expected,
+                                  inspected_actual);
+        message = cut_append_diff(message, inspected_expected, inspected_actual);
+        g_free(inspected_expected);
+        g_free(inspected_actual);
+        cut_test_fail(message);
+    }
+}
+
+void
+gcut_assert_equal_hash_table_string_string_helper (GHashTable  *expected,
+                                                   GHashTable  *actual,
+                                                   const gchar *expression_expected,
+                                                   const gchar *expression_actual)
 {
     if (gcut_hash_table_string_equal(expected, actual)) {
         cut_test_pass();
@@ -290,15 +347,13 @@ gcut_assert_equal_hash_table_string_string_helper (
         message = cut_append_diff(message, inspected_expected, inspected_actual);
         g_free(inspected_expected);
         g_free(inspected_actual);
-        cut_test_fail_va_list(message, user_message_format);
+        cut_test_fail(message);
     }
 }
 
 void
 gcut_assert_error_helper (GError         *error,
-                          const gchar    *expression_error,
-                          const gchar    *user_message_format,
-                          ...)
+                          const gchar    *expression_error)
 {
     if (error == NULL) {
         cut_test_pass();
@@ -306,11 +361,10 @@ gcut_assert_error_helper (GError         *error,
         const gchar *inspected;
 
         inspected = cut_take_string(gcut_error_inspect(error));
-        g_error_free(error);
-        cut_test_fail_va_list(cut_take_printf("expected: <%s> is NULL\n"
-                                              "  actual: <%s>",
-                                              expression_error, inspected),
-                              user_message_format);
+        g_clear_error(&error);
+        cut_test_fail(cut_take_printf("expected: <%s> is NULL\n"
+                                      "  actual: <%s>",
+                                      expression_error, inspected));
     }
 }
 
@@ -318,9 +372,7 @@ void
 gcut_assert_equal_error_helper (const GError   *expected,
                                 const GError   *actual,
                                 const gchar    *expression_expected,
-                                const gchar    *expression_actual,
-                                const gchar    *user_message_format,
-                                ...)
+                                const gchar    *expression_actual)
 {
     if (gcut_error_equal(expected, actual)) {
         cut_test_pass();
@@ -341,19 +393,52 @@ gcut_assert_equal_error_helper (const GError   *expected,
         if (expected && actual)
             message = cut_append_diff(message,
                                       inspected_expected, inspected_actual);
-        cut_test_fail_va_list(message, user_message_format);
+        cut_test_fail(message);
     }
 }
 
+void
+gcut_assert_remove_path_helper (const gchar *path, ...)
+{
+    GError *remove_path_error = NULL;
+    gchar *full_path;
+    va_list args;
+
+    va_start(args, path);
+    full_path = cut_utils_build_path_va_list(path, args);
+    va_end(args);
+
+    cut_utils_remove_path_recursive(full_path, &remove_path_error);
+    gcut_assert_error_helper(remove_path_error, cut_take_string(full_path));
+}
+
+void
+gcut_assert_equal_time_val_helper (GTimeVal       *expected,
+                                   GTimeVal       *actual,
+                                   const gchar    *expression_expected,
+                                   const gchar    *expression_actual)
+{
+    const gchar *expected_string = NULL;
+    const gchar *actual_string = NULL;
+
+    if (expected)
+        expected_string = cut_take_string(g_time_val_to_iso8601(expected));
+    if (actual)
+        actual_string = cut_take_string(g_time_val_to_iso8601(actual));
+
+    cut_assert_equal_string_helper(expected_string,
+                                   actual_string,
+                                   expression_expected,
+                                   expression_actual);
+}
+
 void
 gcut_assert_equal_enum_helper (GType           enum_type,
                                gint            expected,
                                gint            actual,
                                const gchar    *expression_enum_type,
                                const gchar    *expression_expected,
-                               const gchar    *expression_actual,
-                               const gchar    *user_message_format,
-                               ...)
+                               const gchar    *expression_actual)
 {
     if (expected == actual) {
         cut_test_pass();
@@ -375,7 +460,7 @@ gcut_assert_equal_enum_helper (GType           enum_type,
                                   inspected_expected,
                                   inspected_actual);
         message = cut_append_diff(message, inspected_expected, inspected_actual);
-        cut_test_fail_va_list(message, user_message_format);
+        cut_test_fail(message);
     }
 }
 
@@ -385,9 +470,7 @@ gcut_assert_equal_flags_helper (GType           flags_type,
                                 gint            actual,
                                 const gchar    *expression_flags_type,
                                 const gchar    *expression_expected,
-                                const gchar    *expression_actual,
-                                const gchar    *user_message_format,
-                                ...)
+                                const gchar    *expression_actual)
 {
     if (expected == actual) {
         cut_test_pass();
@@ -409,7 +492,7 @@ gcut_assert_equal_flags_helper (GType           flags_type,
                                   inspected_expected,
                                   inspected_actual);
         message = cut_append_diff(message, inspected_expected, inspected_actual);
-        cut_test_fail_va_list(message, user_message_format);
+        cut_test_fail(message);
     }
 }
 
@@ -419,9 +502,7 @@ gcut_assert_equal_object_helper (GObject        *expected,
                                  GEqualFunc      equal_function,
                                  const gchar    *expression_expected,
                                  const gchar    *expression_actual,
-                                 const gchar    *expression_equal_function,
-                                 const gchar    *user_message_format,
-                                 ...)
+                                 const gchar    *expression_equal_function)
 {
     if (gcut_object_equal(expected, actual, equal_function)) {
         cut_test_pass();
@@ -458,7 +539,7 @@ gcut_assert_equal_object_helper (GObject        *expected,
         g_free(inspected_actual);
         g_string_free(message, TRUE);
 
-        cut_test_fail_va_list(fail_message, user_message_format);
+        cut_test_fail(fail_message);
     }
 }
 
@@ -466,21 +547,17 @@ void
 gcut_assert_equal_int64_helper (gint64          expected,
                                 gint64          actual,
                                 const char     *expression_expected,
-                                const char     *expression_actual,
-                                const char     *user_message_format,
-                                ...)
+                                const char     *expression_actual)
 {
     if (expected == actual) {
         cut_test_pass();
     } else {
-        cut_test_fail_va_list(
-            cut_take_printf("<%s == %s>\n"
-                            "expected: <%" G_GINT64_FORMAT ">\n"
-                            "  actual: <%" G_GINT64_FORMAT ">",
-                            expression_expected,
-                            expression_actual,
-                            expected, actual),
-            user_message_format);
+        cut_test_fail(cut_take_printf("<%s == %s>\n"
+                                      "expected: <%" G_GINT64_FORMAT ">\n"
+                                      "  actual: <%" G_GINT64_FORMAT ">",
+                                      expression_expected,
+                                      expression_actual,
+                                      expected, actual));
     }
 }
 
@@ -488,21 +565,58 @@ void
 gcut_assert_equal_uint64_helper (guint64         expected,
                                  guint64         actual,
                                  const char     *expression_expected,
-                                 const char     *expression_actual,
-                                 const char     *user_message_format,
-                                 ...)
+                                 const char     *expression_actual)
 {
     if (expected == actual) {
         cut_test_pass();
     } else {
-        cut_test_fail_va_list(
-            cut_take_printf("<%s == %s>\n"
-                            "expected: <%" G_GUINT64_FORMAT ">\n"
-                            "  actual: <%" G_GUINT64_FORMAT ">",
-                            expression_expected,
-                            expression_actual,
-                            expected, actual),
-            user_message_format);
+        cut_test_fail(cut_take_printf("<%s == %s>\n"
+                                      "expected: <%" G_GUINT64_FORMAT ">\n"
+                                      "  actual: <%" G_GUINT64_FORMAT ">",
+                                      expression_expected,
+                                      expression_actual,
+                                      expected, actual));
+    }
+}
+
+#ifdef G_OS_WIN32
+#define GCUT_GPID_FORMAT "p"
+#else
+#define GCUT_GPID_FORMAT "d"
+#endif
+void
+gcut_assert_equal_pid_helper (GPid            expected,
+                              GPid            actual,
+                              const char     *expression_expected,
+                              const char     *expression_actual)
+{
+    if (expected == actual) {
+        cut_test_pass();
+    } else {
+        cut_test_fail(cut_take_printf("<%s == %s>\n"
+                                      "expected: <%" GCUT_GPID_FORMAT ">\n"
+                                      "  actual: <%" GCUT_GPID_FORMAT ">",
+                                      expression_expected,
+                                      expression_actual,
+                                      expected, actual));
+    }
+}
+
+void
+gcut_assert_not_equal_pid_helper (GPid            expected,
+                                  GPid            actual,
+                                  const char     *expression_expected,
+                                  const char     *expression_actual)
+{
+    if (expected != actual) {
+        cut_test_pass();
+    } else {
+        cut_test_fail(cut_take_printf("<%s != %s>\n"
+                                      "expected: <%" GCUT_GPID_FORMAT ">\n"
+                                      "  actual: <%" GCUT_GPID_FORMAT ">",
+                                      expression_expected,
+                                      expression_actual,
+                                      expected, actual));
     }
 }
 
